@@ -1,9 +1,10 @@
 # BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
 
+import copy
+
 import awkward as ak
 from awkward._v2.contents.content import Content, unset
 from awkward._v2.forms.emptyform import EmptyForm
-from awkward._v2.forms.form import _parameters_equal
 
 np = ak.nplike.NumpyMetadata.instance()
 numpy = ak.nplike.Numpy.instance()
@@ -25,6 +26,15 @@ class EmptyArray(Content):
             self._nplike if nplike is unset else nplike,
         )
 
+    def __copy__(self):
+        return self.copy()
+
+    def __deepcopy__(self, memo):
+        return self.copy(
+            identifier=copy.deepcopy(self._identifier, memo),
+            parameters=copy.deepcopy(self._parameters, memo),
+        )
+
     def __init__(self, identifier=None, parameters=None, nplike=None):
         if nplike is None:
             nplike = numpy
@@ -184,9 +194,7 @@ class EmptyArray(Content):
             offsets = ak._v2.index.Index64.zeros(1, self._nplike)
             return (offsets, EmptyArray(None, self._parameters, self._nplike))
 
-    def mergeable(self, other, mergebool):
-        if not _parameters_equal(self._parameters, other._parameters):
-            return False
+    def _mergeable(self, other, mergebool):
         return True
 
     def mergemany(self, others):
