@@ -1,7 +1,5 @@
 # BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
 
-from __future__ import absolute_import
-
 import awkward as ak
 import numpy as np
 
@@ -10,7 +8,7 @@ def v1v2_id_equal(v1, v2):
     if v1 is None and v2 is None:
         return True
     else:
-        raise NotImplementedError("Identities/Identifier equality")
+        raise ak._v2._util.error(NotImplementedError("Identities/Identifier equality"))
 
 
 def v1v2_equal(v1, v2):
@@ -56,17 +54,14 @@ def v1v2_equal(v1, v2):
             and v1_stops.dtype == v2_stops.dtype
         )
 
-    elif (
-        isinstance(
-            v1,
-            (
-                ak.layout.ListOffsetArray32,
-                ak.layout.ListOffsetArrayU32,
-                ak.layout.ListOffsetArray64,
-            ),
-        )
-        and isinstance(v2, ak._v2.contents.ListOffsetArray)
-    ):
+    elif isinstance(
+        v1,
+        (
+            ak.layout.ListOffsetArray32,
+            ak.layout.ListOffsetArrayU32,
+            ak.layout.ListOffsetArray64,
+        ),
+    ) and isinstance(v2, ak._v2.contents.ListOffsetArray):
         v1_offsets = np.asarray(v1.offsets)
         v2_offsets = v2.offsets.data
         return (
@@ -83,17 +78,14 @@ def v1v2_equal(v1, v2):
             for i in range(v1.numfields)
         )
 
-    elif (
-        isinstance(
-            v1,
-            (
-                ak.layout.IndexedArray32,
-                ak.layout.IndexedArrayU32,
-                ak.layout.IndexedArray64,
-            ),
-        )
-        and isinstance(v2, ak._v2.contents.IndexedArray)
-    ):
+    elif isinstance(
+        v1,
+        (
+            ak.layout.IndexedArray32,
+            ak.layout.IndexedArrayU32,
+            ak.layout.IndexedArray64,
+        ),
+    ) and isinstance(v2, ak._v2.contents.IndexedArray):
         v1_index = np.asarray(v1.index)
         v2_index = v2.index.data
         return (
@@ -143,17 +135,14 @@ def v1v2_equal(v1, v2):
     ):
         return v1v2_equal(v1.content, v2.content)
 
-    elif (
-        isinstance(
-            v1,
-            (
-                ak.layout.UnionArray8_32,
-                ak.layout.UnionArray8_U32,
-                ak.layout.UnionArray8_64,
-            ),
-        )
-        and isinstance(v2, ak._v2.contents.UnionArray)
-    ):
+    elif isinstance(
+        v1,
+        (
+            ak.layout.UnionArray8_32,
+            ak.layout.UnionArray8_U32,
+            ak.layout.UnionArray8_64,
+        ),
+    ) and isinstance(v2, ak._v2.contents.UnionArray):
         v1_tags = np.asarray(v1.tags)
         v2_tags = v2.tags.data
         v1_index = np.asarray(v1.index)
@@ -171,17 +160,17 @@ def v1v2_equal(v1, v2):
     elif isinstance(v1, ak.layout.VirtualArray) and isinstance(
         v2, ak._v2.contents.VirtualArray
     ):
-        raise AssertionError("VirtualArray is v1 only")
+        raise ak._v2._util.error(AssertionError("VirtualArray is v1 only"))
 
     else:
-        raise AssertionError("{0} vs {1}".format(type(v1), type(v2)))
+        raise ak._v2._util.error(AssertionError(f"{type(v1)} vs {type(v2)}"))
 
 
 def v1_to_v2_id(v1):
     if v1 is None:
         return None
     else:
-        raise NotImplementedError("Identities to Identifier")
+        raise ak._v2._util.error(NotImplementedError("Identities to Identifier"))
 
 
 def fix(array):
@@ -217,7 +206,7 @@ def v1_to_v2(v1):
 
     elif isinstance(v1, ak.layout.NumpyArray):
         primitive = v1.form.primitive
-        if primitive == "datetime64" or primitive == "timedelta64":
+        if primitive in {"datetime64", "timedelta64"}:
             return ak._v2.contents.NumpyArray(
                 fix(np.asarray(v1.view_int64)).view(np.dtype(v1.format)),
                 identifier=v1_to_v2_id(v1.identities),
@@ -343,17 +332,17 @@ def v1_to_v2(v1):
         )
 
     elif isinstance(v1, ak.layout.VirtualArray):
-        raise AssertionError("VirtualArray is v1 only")
+        raise ak._v2._util.error(AssertionError("VirtualArray is v1 only"))
 
     else:
-        raise AssertionError(type(v1))
+        raise ak._v2._util.error(AssertionError(type(v1)))
 
 
 def v2_to_v1_id(v1):
     if v1 is None:
         return None
     else:
-        raise NotImplementedError("Identifier to Identities")
+        raise ak._v2._util.error(NotImplementedError("Identifier to Identities"))
 
 
 def v2_to_v1(v2):
@@ -503,7 +492,7 @@ def v2_to_v1(v2):
         )
 
     else:
-        raise AssertionError(type(v2))
+        raise ak._v2._util.error(AssertionError(type(v2)))
 
 
 def v1_to_v2_index(v1):
@@ -534,4 +523,4 @@ def v1_to_v2_index(v1):
         return ak._v2.index.Index64(np.asarray(v1))
 
     else:
-        raise AssertionError(type(v1))
+        raise ak._v2._util.error(AssertionError(type(v1)))
